# Check GoogleTest shard test crashes are handled.

# RUN: not %{lit} -v %{inputs}/googletest-crash | FileCheck %s

# CHECK: -- Testing:
# CHECK: FAIL: googletest-crash :: [[PATH:[Dd]ummy[Ss]ub[Dd]ir/]][[FILE:OneTest\.py]]/0
# CHECK: *** TEST 'googletest-crash :: [[PATH]][[FILE]]/0{{.*}} FAILED ***
# CHECK-NEXT: Script(shard):
# CHECK-NEXT: --
# CHECK-NEXT: GTEST_OUTPUT=json:[[JSON:[^[:space:]]*\.json]] GTEST_SHUFFLE=0 GTEST_TOTAL_SHARDS={{[1-6]}} GTEST_SHARD_INDEX=0 {{.*}}[[FILE]]
# CHECK-NEXT: --
# CHECK-EMPTY:
# CHECK-NEXT: [----------] 4 test from FirstTest
# CHECK-NEXT: [ RUN      ] FirstTest.subTestA
# CHECK-NEXT: [       OK ] FirstTest.subTestA (18 ms)
# CHECK-NEXT: [ RUN      ] FirstTest.subTestB
# CHECK-NEXT: I am about to crash
# CHECK-EMPTY:
# CHECK-NEXT: --
# CHECK-NEXT: exit:
# CHECK-NEXT: --
# CHECK-NEXT: shard JSON output does not exist: [[JSON]]
# CHECK-NEXT: ***
# CHECK: Failed Tests (1):
# CHECK-NEXT:   googletest-crash :: [[PATH]][[FILE]]/0/{{[1-6]}}
# CHECK: Failed{{ *}}: 1



# Check that XFAILing works via command line or env var.

# RUN: %{lit} --xfail 'false.txt;false2.txt;top-level-suite :: b :: test.txt' \
# RUN:   --xfail-not 'true-xfail.txt;top-level-suite :: a :: test-xfail.txt' \
# RUN:   %{inputs}/xfail-cl \
# RUN: | FileCheck --check-prefix=CHECK-FILTER %s

# RUN: env LIT_XFAIL='false.txt;false2.txt;top-level-suite :: b :: test.txt' \
# RUN:   LIT_XFAIL_NOT='true-xfail.txt;top-level-suite :: a :: test-xfail.txt' \
# RUN: %{lit} %{inputs}/xfail-cl \
# RUN: | FileCheck --check-prefix=CHECK-FILTER %s

# Check that --xfail-not and LIT_XFAIL_NOT always have precedence.

# RUN: env LIT_XFAIL=true-xfail.txt \
# RUN: %{lit} --xfail true-xfail.txt --xfail-not true-xfail.txt \
# RUN:   --xfail true-xfail.txt %{inputs}/xfail-cl/true-xfail.txt \
# RUN: | FileCheck --check-prefix=CHECK-OVERRIDE %s

# RUN: env LIT_XFAIL_NOT=true-xfail.txt LIT_XFAIL=true-xfail.txt \
# RUN: %{lit} --xfail true-xfail.txt %{inputs}/xfail-cl/true-xfail.txt \
# RUN: | FileCheck --check-prefix=CHECK-OVERRIDE %s

# END.

# CHECK-FILTER: Testing: 10 tests, {{[0-9]*}} workers
# CHECK-FILTER-DAG: {{^}}PASS: top-level-suite :: a :: test.txt
# CHECK-FILTER-DAG: {{^}}XFAIL: top-level-suite :: b :: test.txt
# CHECK-FILTER-DAG: {{^}}XFAIL: top-level-suite :: a :: false.txt
# CHECK-FILTER-DAG: {{^}}XFAIL: top-level-suite :: b :: false.txt
# CHECK-FILTER-DAG: {{^}}XFAIL: top-level-suite :: false.txt
# CHECK-FILTER-DAG: {{^}}XFAIL: top-level-suite :: false2.txt
# CHECK-FILTER-DAG: {{^}}PASS: top-level-suite :: true.txt
# CHECK-FILTER-DAG: {{^}}PASS: top-level-suite :: true-xfail.txt
# CHECK-FILTER-DAG: {{^}}PASS: top-level-suite :: a :: test-xfail.txt
# CHECK-FILTER-DAG: {{^}}XFAIL: top-level-suite :: b :: test-xfail.txt

# CHECK-OVERRIDE: Testing: 1 tests, {{[0-9]*}} workers
# CHECK-OVERRIDE: {{^}}PASS: top-level-suite :: true-xfail.txt



# Check the GoogleTest format support command wrappers.

# RUN: %{lit} -v %{inputs}/googletest-cmd-wrapper | FileCheck %s

# CHECK: -- Testing:
# CHECK-NEXT: PASS: googletest-cmd-wrapper :: DummySubDir/OneTest.exe/0/1 (1 of 1)
# CHECK: Passed: 1



# Check that --ignore-fail produces exit status 0 despite various kinds of
# test failures but doesn't otherwise suppress those failures.

# RUN: not %{lit} %{inputs}/ignore-fail | FileCheck %s
# RUN: %{lit} --ignore-fail %{inputs}/ignore-fail | FileCheck %s

# END.

# CHECK-DAG: FAIL: ignore-fail :: fail.txt
# CHECK-DAG: UNRESOLVED: ignore-fail :: unresolved.txt
# CHECK-DAG: XFAIL: ignore-fail :: xfail.txt
# CHECK-DAG: XPASS: ignore-fail :: xpass.txt

#      CHECK: Testing Time:
# CHECK-NEXT:   Expectedly Failed : 1
# CHECK-NEXT:   Unresolved : 1
# CHECK-NEXT:   Failed : 1
# CHECK-NEXT:   Unexpectedly Passed: 1
#  CHECK-NOT: {{.}}



#!/usr/bin/env python

import os
import sys

if len(sys.argv) == 3 and sys.argv[1] == "--gtest_list_tests":
    if sys.argv[2] != '--gtest_filter=-*DISABLED_*':
        raise ValueError("unexpected argument: %s" % (sys.argv[2]))
    print("""\
FirstTest.
  subTestA
  subTestB
  subTestC
  subTestD
ParameterizedTest/0.
  subTest
ParameterizedTest/1.
  subTest""")
    sys.exit(0)
elif len(sys.argv) != 1:
    # sharding and json output are specified using environment variables
    raise ValueError("unexpected argument: %r" % (' '.join(sys.argv[1:])))

for e in ['GTEST_TOTAL_SHARDS', 'GTEST_SHARD_INDEX', 'GTEST_OUTPUT']:
    if e not in os.environ:
        raise ValueError("missing environment variables: " + e)

if not os.environ['GTEST_OUTPUT'].startswith('json:'):
    raise ValueError("must emit json output: " + os.environ['GTEST_OUTPUT'])

dummy_output = """\
{
"testsuites": [
]
}"""

if os.environ['GTEST_SHARD_INDEX'] == '0':
    print("""\
[----------] 4 test from FirstTest
[ RUN      ] FirstTest.subTestA
[       OK ] FirstTest.subTestA (18 ms)
[ RUN      ] FirstTest.subTestB""", flush=True)
    print('I am about to crash', file=sys.stderr, flush=True)
    exit_code = 1
else:
    json_filename = os.environ['GTEST_OUTPUT'].split(':', 1)[1]
    with open(json_filename, 'w', encoding='utf-8') as f:
        f.write(dummy_output)
    exit_code = 0

sys.exit(exit_code)



# Check the pushd and popd commands

# RUN: not %{lit} -a -v %{inputs}/shtest-pushd-popd \
# RUN: | FileCheck -match-full-lines %s
#
# END.

# CHECK: -- Testing: 4 tests{{.*}}

# CHECK: FAIL: shtest-pushd-popd :: popd-args.txt ({{[^)]*}})
# CHECK: $ "popd" "invalid"
# CHECK: 'popd' does not support arguments

# CHECK: FAIL: shtest-pushd-popd :: popd-no-stack.txt ({{[^)]*}})
# CHECK: $ "popd"
# CHECK: popd: directory stack empty

# CHECK: FAIL: shtest-pushd-popd :: pushd-too-many-args.txt ({{[^)]*}})
# CHECK: $ "pushd" "a" "b"
# CHECK: 'pushd' supports only one argument

# CHECK: Passed:  1
# CHECK: Failed:  3
# CHECK-NOT: {{.}}



# Check the various features of the GoogleTest format.

# RUN: not %{lit} -v --order=random %{inputs}/googletest-format > %t.out
# FIXME: Temporarily dump test output so we can debug failing tests on
# buildbots.
# RUN: cat %t.out
# RUN: FileCheck < %t.out %s
#
# END.

# CHECK: -- Testing:
# CHECK: FAIL: googletest-format :: [[PATH:[Dd]ummy[Ss]ub[Dd]ir/]][[FILE:OneTest\.py]]/0
# CHECK: *** TEST 'googletest-format :: [[PATH]][[FILE]]/0{{.*}} FAILED ***
# CHECK-NEXT: Script(shard):
# CHECK-NEXT: --
# CHECK-NEXT: GTEST_OUTPUT=json:{{[^[:space:]]*}} GTEST_SHUFFLE=1 GTEST_TOTAL_SHARDS={{[1-6]}} GTEST_SHARD_INDEX=0 GTEST_RANDOM_SEED=123 {{.*}}[[FILE]]
# CHECK-NEXT: --
# CHECK-EMPTY:
# CHECK-NEXT: Script:
# CHECK-NEXT: --
# CHECK-NEXT: [[FILE]] --gtest_filter=FirstTest.subTestB
# CHECK-NEXT: --
# CHECK-NEXT: I am subTest B output
# CHECK-EMPTY:
# CHECK-NEXT: I am subTest B, I FAIL
# CHECK-NEXT: And I have two lines of output
# CHECK-EMPTY:
# CHECK: Script:
# CHECK-NEXT: --
# CHECK-NEXT: [[FILE]] --gtest_filter=FirstTest.subTestD
# CHECK-NEXT: --
# CHECK-NEXT: unresolved test result
# CHECK: ***
# CHECK: Unresolved Tests (1):
# CHECK-NEXT:   googletest-format :: [[PATH]][[FILE]]/FirstTest/subTestD
# CHECK: ***
# CHECK-NEXT: Failed Tests (1):
# CHECK-NEXT:   googletest-format :: [[PATH]][[FILE]]/FirstTest/subTestB
# CHECK: Skipped{{ *}}: 1
# CHECK: Passed{{ *}}: 3
# CHECK: Unresolved{{ *}}: 1
# CHECK: Failed{{ *}}: 1



# Check GTEST_TOTAL_SHARDS and GTEST_SHARD_INDEX environment variabls are
# respected when using the googletest formatter.

# RUN: env GTEST_TOTAL_SHARDS=1 GTEST_SHARD_INDEX=0 \
# RUN: not %{lit} -v --order=random %{inputs}/googletest-format-respect-gtest-sharding-env-vars > %t.out
# FIXME: Temporarily dump test output so we can debug failing tests on
# buildbots.
# RUN: cat %t.out
# RUN: FileCheck < %t.out %s
#
# END.

# CHECK: -- Testing:
# CHECK: FAIL: googletest-format :: [[PATH:[Dd]ummy[Ss]ub[Dd]ir/]][[FILE:OneTest\.py]]/0
# CHECK: *** TEST 'googletest-format :: [[PATH]][[FILE]]/0{{.*}} FAILED ***
# CHECK-NEXT: Script(shard):
# CHECK-NEXT: --
# CHECK-NEXT: GTEST_OUTPUT=json:{{[^[:space:]]*}} GTEST_SHUFFLE=1 GTEST_TOTAL_SHARDS=1 GTEST_SHARD_INDEX=0 GTEST_RANDOM_SEED=123 {{.*}}[[FILE]]



# RUN: %{lit} -j 1 -v %{inputs}/test-data --resultdb-output %t.results.out > %t.out
# RUN: FileCheck < %t.results.out %s
# RUN: %{lit} -j 1 -v %{inputs}/googletest-cmd-wrapper --resultdb-output %t.results-unit.out > %t.out
# RUN: FileCheck < %t.results-unit.out --check-prefix=UNIT %s

# CHECK: {
# CHECK: "__version__"
# CHECK: "elapsed"
# CHECK-NEXT: "tests": [
# CHECK-NEXT:   {
# CHECK-NEXT:      "artifacts": {
# CHECK-NEXT:        "artifact-content-in-request": {
# CHECK-NEXT:          "contents": "VGVzdCBwYXNzZWQu"
# CHECK-NEXT:        }
# CHECK-NEXT:      },
# CHECK-NEXT:      "duration"
# CHECK-NEXT:      "expected": true,
# CHECK-NEXT:      "start_time"
# CHECK-NEXT:      "status": "PASS",
# CHECK-NEXT:      "summary_html": "<p><text-artifact artifact-id=\"artifact-content-in-request\"></p>",
# CHECK-NEXT:      "testId": "test-data :: metrics.ini"
# CHECK-NEXT:    }
# CHECK-NEXT: ]
# CHECK-NEXT: }

# UNIT: {
# UNIT: "__version__"
# UNIT: "elapsed"
# UNIT-NEXT: "tests": [
# UNIT-NEXT:   {
# UNIT-NEXT:     "artifacts": {
# UNIT-NEXT:       "artifact-content-in-request": {
# UNIT-NEXT:         "contents": ""
# UNIT-NEXT:       }
# UNIT-NEXT:     },
# UNIT-NEXT:     "duration"
# UNIT-NEXT:     "expected": true,
# UNIT-NEXT:     "start_time"
# UNIT-NEXT:     "status": "PASS",
# UNIT-NEXT:     "summary_html": "<p><text-artifact artifact-id=\"artifact-content-in-request\"></p>",
# UNIT-NEXT:     "testId": "googletest-cmd-wrapper :: DummySubDir/OneTest.exe/FirstTest/subTestA"
# UNIT-NEXT:   }
# UNIT-NEXT: ]
# UNIT-NEXT: }



from __future__ import absolute_import
import json
import math
import os
import shlex
import subprocess
import sys

import lit.Test
import lit.TestRunner
import lit.util
from .base import TestFormat

kIsWindows = sys.platform in ['win32', 'cygwin']

class GoogleTest(TestFormat):
    def __init__(self, test_sub_dirs, test_suffix, run_under = []):
        self.seen_executables = set()
        self.test_sub_dirs = str(test_sub_dirs).split(';')

        # On Windows, assume tests will also end in '.exe'.
        exe_suffix = str(test_suffix)
        if kIsWindows:
            exe_suffix += '.exe'

        # Also check for .py files for testing purposes.
        self.test_suffixes = {exe_suffix, test_suffix + '.py'}
        self.run_under = run_under

    def get_num_tests(self, path, litConfig, localConfig):
        list_test_cmd = self.prepareCmd(
            [path, '--gtest_list_tests', '--gtest_filter=-*DISABLED_*'])
        try:
            out = subprocess.check_output(list_test_cmd,
                                          env=localConfig.environment)
        except subprocess.CalledProcessError as exc:
            litConfig.warning(
                "unable to discover google-tests in %r: %s. Process output: %s"
                % (path, sys.exc_info()[1], exc.output))
            return None
        return sum(
            map(lambda line: lit.util.to_string(line).startswith('  '),
                out.splitlines(False)))

    def getTestsInDirectory(self, testSuite, path_in_suite, litConfig,
                            localConfig):
        init_shard_size = 512  # number of tests in a shard
        core_count = lit.util.usable_core_count()
        source_path = testSuite.getSourcePath(path_in_suite)
        for subdir in self.test_sub_dirs:
            dir_path = os.path.join(source_path, subdir)
            if not os.path.isdir(dir_path):
                continue
            for fn in lit.util.listdir_files(dir_path,
                                             suffixes=self.test_suffixes):
                # Discover the tests in this executable.
                execpath = os.path.join(source_path, subdir, fn)
                if execpath in self.seen_executables:
                    litConfig.warning(
                        "Skip adding %r since it has been added to the test pool" % execpath)
                    continue
                else:
                    self.seen_executables.add(execpath)
                num_tests = self.get_num_tests(execpath, litConfig,
                                               localConfig)
                if num_tests is not None:
                    # Compute the number of shards.
                    shard_size = init_shard_size
                    nshard = int(math.ceil(num_tests / shard_size))
                    while nshard < core_count and shard_size > 1:
                        shard_size = shard_size // 2
                        nshard = int(math.ceil(num_tests / shard_size))

                    # Create one lit test for each shard.
                    for idx in range(nshard):
                        testPath = path_in_suite + (subdir, fn, str(idx),
                                                    str(nshard))
                        json_file = '-'.join([
                            execpath, testSuite.config.name,
                            str(os.getpid()),
                            str(idx),
                            str(nshard)
                        ]) + '.json'
                        yield lit.Test.Test(testSuite,
                                            testPath,
                                            localConfig,
                                            file_path=execpath,
                                            gtest_json_file=json_file)
                else:
                    # This doesn't look like a valid gtest file.  This can
                    # have a number of causes, none of them good.  For
                    # instance, we could have created a broken executable.
                    # Alternatively, someone has cruft in their test
                    # directory.  If we don't return a test here, then no
                    # failures will get reported, so return a dummy test name
                    # so that the failure is reported later.
                    testPath = path_in_suite + (
                        subdir, fn, 'failed_to_discover_tests_from_gtest')
                    yield lit.Test.Test(testSuite,
                                        testPath,
                                        localConfig,
                                        file_path=execpath)

    def execute(self, test, litConfig):
        if test.gtest_json_file is None:
            return lit.Test.FAIL, ''

        testPath,testName = os.path.split(test.getSourcePath())
        while not os.path.exists(testPath):
            # Handle GTest parametrized and typed tests, whose name includes
            # some '/'s.
            testPath, namePrefix = os.path.split(testPath)
            testName = namePrefix + '/' + testName

        testName,total_shards = os.path.split(testName)
        testName,shard_idx = os.path.split(testName)
        from lit.cl_arguments import TestOrder
        use_shuffle = TestOrder(litConfig.order) == TestOrder.RANDOM
        shard_env = {
            'GTEST_OUTPUT': 'json:' + test.gtest_json_file,
            'GTEST_SHUFFLE': '1' if use_shuffle else '0',
            'GTEST_TOTAL_SHARDS': os.environ.get("GTEST_TOTAL_SHARDS", total_shards),
            'GTEST_SHARD_INDEX': os.environ.get("GTEST_SHARD_INDEX", shard_idx)
        }
        test.config.environment.update(shard_env)

        cmd = [testPath]
        cmd = self.prepareCmd(cmd)
        if litConfig.useValgrind:
            cmd = litConfig.valgrindArgs + cmd

        if litConfig.noExecute:
            return lit.Test.PASS, ''

        def get_shard_header(shard_env):
            shard_envs = ' '.join([k + '=' + v for k, v in shard_env.items()])
            return f"Script(shard):\n--\n%s %s\n--\n" % (shard_envs, ' '.join(cmd))

        shard_header = get_shard_header(shard_env)

        try:
            out, _, exitCode = lit.util.executeCommand(
                cmd, env=test.config.environment,
                timeout=litConfig.maxIndividualTestTime, redirect_stderr=True)
        except lit.util.ExecuteCommandTimeoutException as e:
            stream_msg = f"\n{e.out}\n--\nexit: {e.exitCode}\n--\n"
            return (lit.Test.TIMEOUT, f'{shard_header}{stream_msg}Reached '
                    f'timeout of {litConfig.maxIndividualTestTime} seconds')

        if not os.path.exists(test.gtest_json_file):
            errmsg = f"shard JSON output does not exist: %s" % (
                test.gtest_json_file)
            stream_msg = f"\n{out}\n--\nexit: {exitCode}\n--\n"
            return lit.Test.FAIL, shard_header + stream_msg + errmsg

        if exitCode == 0:
            return lit.Test.PASS, ''

        def get_test_stdout(test_name):
            res = []
            header = f'[ RUN      ] ' + test_name
            footer = f'[  FAILED  ] ' + test_name
            in_range = False
            for l in out.splitlines():
                if l.startswith(header):
                    in_range = True
                elif l.startswith(footer):
                    return f'' if len(res) == 0 else '\n'.join(res)
                elif in_range:
                    res.append(l)
            assert False, f'gtest did not report the result for ' + test_name

        found_failed_test = False

        with open(test.gtest_json_file, encoding='utf-8') as f:
            jf = json.load(f)

            if use_shuffle:
                shard_env['GTEST_RANDOM_SEED'] = str(jf['random_seed'])
            output = get_shard_header(shard_env) + '\n'

            for testcase in jf['testsuites']:
                for testinfo in testcase['testsuite']:
                    result = testinfo['result']
                    if result == 'SUPPRESSED' or result == 'SKIPPED':
                        continue
                    testname = testcase['name'] + '.' + testinfo['name']
                    header = f"Script:\n--\n%s --gtest_filter=%s\n--\n" % (
                        ' '.join(cmd), testname)
                    if 'failures' in testinfo:
                        found_failed_test = True
                        output += header
                        test_out = get_test_stdout(testname)
                        if test_out:
                            output += test_out + '\n\n'
                        for fail in testinfo['failures']:
                            output += fail['failure'] + '\n'
                        output += '\n'
                    elif result != 'COMPLETED':
                        output += header
                        output += 'unresolved test result\n'

        # In some situations, like running tests with sanitizers, all test passes but
        # the shard could still fail due to memory issues.
        if not found_failed_test:
            output += f"\n{out}\n--\nexit: {exitCode}\n--\n"

        return lit.Test.FAIL, output

    def prepareCmd(self, cmd):
        """Insert interpreter if needed.

        It inserts the python exe into the command if cmd[0] ends in .py or caller
        specified run_under.
        We cannot rely on the system to interpret shebang lines for us on
        Windows, so add the python executable to the command if this is a .py
        script.
        """
        if cmd[0].endswith('.py'):
            cmd = [sys.executable] + cmd
        if self.run_under:
            if isinstance(self.run_under, list):
                cmd = self.run_under + cmd
            else:
                cmd = shlex.split(self.run_under) + cmd
        return cmd

    @staticmethod
    def post_process_shard_results(selected_tests, discovered_tests):
        def remove_gtest(tests):
            return [t for t in tests if t.gtest_json_file is None]

        discovered_tests = remove_gtest(discovered_tests)
        gtests = [t for t in selected_tests if t.gtest_json_file]
        selected_tests = remove_gtest(selected_tests)
        for test in gtests:
            # In case gtest has bugs such that no JSON file was emitted.
            if not os.path.exists(test.gtest_json_file):
                selected_tests.append(test)
                discovered_tests.append(test)
                continue

            start_time = test.result.start or 0.0

            has_failure_in_shard = False

            # Load json file to retrieve results.
            with open(test.gtest_json_file, encoding='utf-8') as f:
                try:
                    testsuites = json.load(f)['testsuites']
                except json.JSONDecodeError as e:
                    raise RuntimeError("Failed to parse json file: " +
                                       test.gtest_json_file + "\n" + e.doc)
                for testcase in testsuites:
                    for testinfo in testcase['testsuite']:
                        # Ignore disabled tests.
                        if testinfo['result'] == 'SUPPRESSED':
                            continue

                        testPath = test.path_in_suite[:-2] + (testcase['name'],
                                                              testinfo['name'])
                        subtest = lit.Test.Test(test.suite, testPath,
                                                test.config, test.file_path)

                        testname = testcase['name'] + '.' + testinfo['name']
                        header = f"Script:\n--\n%s --gtest_filter=%s\n--\n" % (
                            test.file_path, testname)

                        output = ''
                        if testinfo['result'] == 'SKIPPED':
                            returnCode = lit.Test.SKIPPED
                        elif 'failures' in testinfo:
                            has_failure_in_shard = True
                            returnCode = lit.Test.FAIL
                            output = header
                            for fail in testinfo['failures']:
                                output += fail['failure'] + '\n'
                        elif testinfo['result'] == 'COMPLETED':
                            returnCode = lit.Test.PASS
                        else:
                            returnCode = lit.Test.UNRESOLVED
                            output = header + 'unresolved test result\n'

                        elapsed_time = float(testinfo['time'][:-1])
                        res = lit.Test.Result(returnCode, output, elapsed_time)
                        res.pid = test.result.pid or 0
                        res.start = start_time
                        start_time = start_time + elapsed_time
                        subtest.setResult(res)

                        selected_tests.append(subtest)
                        discovered_tests.append(subtest)
            os.remove(test.gtest_json_file)

            if not has_failure_in_shard and test.isFailure():
                selected_tests.append(test)
                discovered_tests.append(test)

        return selected_tests, discovered_tests



#!/usr/bin/env python

import os
import sys

if len(sys.argv) == 3 and sys.argv[1] == "--gtest_list_tests":
    if sys.argv[2] != '--gtest_filter=-*DISABLED_*':
        raise ValueError("unexpected argument: %s" % (sys.argv[2]))
    print("""\
FirstTest.
  subTestA
  subTestB
  subTestC
  subTestD
ParameterizedTest/0.
  subTest
ParameterizedTest/1.
  subTest""")
    sys.exit(0)
elif len(sys.argv) != 1:
    # sharding and json output are specified using environment variables
    raise ValueError("unexpected argument: %r" % (' '.join(sys.argv[1:])))

for e in ['GTEST_TOTAL_SHARDS', 'GTEST_SHARD_INDEX', 'GTEST_OUTPUT']:
    if e not in os.environ:
        raise ValueError("missing environment variables: " + e)

if not os.environ['GTEST_OUTPUT'].startswith('json:'):
    raise ValueError("must emit json output: " + os.environ['GTEST_OUTPUT'])

output = """\
{
"random_seed": 123,
"testsuites": [
    {
        "name": "FirstTest",
        "testsuite": [
            {
                "name": "subTestA",
                "result": "COMPLETED",
                "time": "0.001s"
            },
            {
                "name": "subTestB",
                "result": "COMPLETED",
                "time": "0.001s",
                "failures": [
                    {
                        "failure": "I am subTest B, I FAIL\\nAnd I have two lines of output",
                        "type": ""
                    }
                ]
            },
            {
                "name": "subTestC",
                "result": "SKIPPED",
                "time": "0.001s"
            },
            {
                "name": "subTestD",
                "result": "UNRESOLVED",
                "time": "0.001s"
            }
        ]
    },
    {
        "name": "ParameterizedTest/0",
        "testsuite": [
            {
                "name": "subTest",
                "result": "COMPLETED",
                "time": "0.001s"
            }
        ]
    },
    {
        "name": "ParameterizedTest/1",
        "testsuite": [
            {
                "name": "subTest",
                "result": "COMPLETED",
                "time": "0.001s"
            }
        ]
    }
]
}"""

dummy_output = """\
{
"testsuites": [
]
}"""

json_filename = os.environ['GTEST_OUTPUT'].split(':', 1)[1]
with open(json_filename, 'w', encoding='utf-8') as f:
    if os.environ['GTEST_SHARD_INDEX'] == '0':
        print('[ RUN      ] FirstTest.subTestB', flush=True)
        print('I am subTest B output', file=sys.stderr, flush=True)
        print('[  FAILED  ] FirstTest.subTestB (8 ms)', flush=True)

        f.write(output)
        exit_code = 1
    else:
        f.write(dummy_output)
        exit_code = 0

sys.exit(exit_code)



# RUN: %{lit} -v --show-all %{inputs}/shtest-if-else/test.txt \
# RUN:    | FileCheck %{inputs}/shtest-if-else/test.txt --match-full-lines

# RUN: not %{lit} -v --show-all %{inputs}/shtest-if-else/test-neg1.txt 2>&1 \
# RUN:    | FileCheck %{inputs}/shtest-if-else/test-neg1.txt

# RUN: not %{lit} -v --show-all %{inputs}/shtest-if-else/test-neg2.txt 2>&1 \
# RUN:    | FileCheck %{inputs}/shtest-if-else/test-neg2.txt

# RUN: not %{lit} -v --show-all %{inputs}/shtest-if-else/test-neg3.txt 2>&1 \
# RUN:    | FileCheck %{inputs}/shtest-if-else/test-neg3.txt

# RUN: not %{lit} -v --show-all %{inputs}/shtest-if-else/test-neg4.txt 2>&1 \
# RUN:    | FileCheck %{inputs}/shtest-if-else/test-neg4.txt



# Check we don't add a GoogleTest binary more than once and issue a warning
# when it happens.

# RUN: %{lit} -v --order=random %{inputs}/googletest-detect-duplicate \
# RUN:                          %{inputs}/googletest-detect-duplicate 2> %t.warn | FileCheck %s
# RUN: FileCheck --check-prefix=CHECK-WARN < %t.warn %s

# CHECK-WARN: warning: Skip adding

# CHECK: -- Testing:
# CHECK: PASS: googletest-detect-duplicate :: [[PATH:[Dd]ummy[Ss]ub[Dd]ir/]][[FILE:OneTest\.py]]/0
# CHECK: Passed{{ *}}: 1



import os
import sys

from setuptools import setup, find_packages

# setuptools expects to be invoked from within the directory of setup.py, but it
# is nice to allow:
#   python path/to/setup.py install
# to work (for scripts, etc.)
os.chdir(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0, ".")

import lit

with open("README.rst", "r", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name = "lit",
    version = lit.__version__,

    author = lit.__author__,
    author_email = lit.__email__,
    url = 'http://llvm.org',
    license = 'Apache-2.0 with LLVM exception',
    license_files = ['LICENSE.TXT'],

    description = "A Software Testing Tool",
    keywords = 'test C++ automatic discovery',
    long_description = long_description,

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Testing',
        ],

    zip_safe = False,
    packages = find_packages(),
    entry_points = {
        'console_scripts': [
            'lit = lit.main:main',
            ],
        }
)



# We're using DEFINE/REDEFINE to help us write tests for DEFINE/REDEFINE.

# RUN: echo '-- Available Tests --' > %t.tests.actual.txt

# DEFINE: %{my-inputs} = %{inputs}/shtest-define

# DEFINE: %{test} =
# DEFINE: %{lit-pre} =
# DEFINE: %{lit-args} =
# DEFINE: %{fc-args} =
# DEFINE: %{run-test} =                                                        \
# DEFINE:   %{lit-pre} %{lit} -va  %{lit-args} %{my-inputs}/%{test} 2>&1 |     \
# DEFINE:     FileCheck -match-full-lines %{fc-args} %{my-inputs}/%{test}      \
# DEFINE:               -dump-input-filter=all -vv -color
# DEFINE: %{record-test} =                                                     \
# DEFINE:   echo '  shtest-define :: %{test}' >> %t.tests.actual.txt
# DEFINE: %{run-and-record-test} = %{run-test} && %{record-test}

# REDEFINE: %{lit-pre} = not
#
# REDEFINE: %{test} = errors/assignment/before-name.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/assignment/between-name-equals.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/assignment/braces-empty.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/assignment/braces-with-dot.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/assignment/braces-with-equals.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/assignment/braces-with-newline.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/assignment/braces-with-number.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/assignment/braces-with-ws.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/assignment/empty.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/assignment/no-equals.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/assignment/no-name.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/assignment/ws-only.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/continuation/empty.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/continuation/end-in-double-backslash.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/continuation/unterminated-define-bad-redefine.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/continuation/unterminated-define-continuation.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/continuation/unterminated-define-redefine.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/continuation/unterminated-define-run.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/continuation/unterminated-define.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/continuation/unterminated-redefine-bad-define.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/continuation/unterminated-redefine-continuation.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/continuation/unterminated-redefine-define.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/continuation/unterminated-redefine-run.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/continuation/unterminated-redefine.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/continuation/unterminated-run-define.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/continuation/unterminated-run-redefine.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/continuation/ws-only.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/defined-check/define-already-by-config.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/defined-check/define-already-by-test.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/defined-check/define-inside-pattern.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/defined-check/define-multiple-exact.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/defined-check/define-multiple-once-exact.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/defined-check/define-prefixes-pattern.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/defined-check/define-suffixes-pattern.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/defined-check/redefine-inside-pattern.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/defined-check/redefine-multiple-exact.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/defined-check/redefine-multiple-once-exact.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/defined-check/redefine-none.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/defined-check/redefine-prefixes-pattern.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/defined-check/redefine-suffixes-pattern.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/location-range.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{test} = errors/no-run.txt
# RUN: %{run-and-record-test}
#
# REDEFINE: %{lit-pre} =

# REDEFINE: %{test} = examples/param-subst.txt
# RUN: %{run-and-record-test}

# REDEFINE: %{test} = expansion-order.txt
# RUN: %{run-and-record-test}

# REDEFINE: %{test} = line-number-substitutions.txt
# RUN: %{run-and-record-test}

# REDEFINE: %{test} = name-chars.txt
# RUN: %{run-and-record-test}

# REDEFINE: %{test} = recursiveExpansionLimit.txt
#
# REDEFINE: %{fc-args} = -check-prefix=CHECK-NON-RECUR
# RUN: %{run-test}
#
# REDEFINE: %{lit-args} = -Drecur=2
# REDEFINE: %{fc-args} = -check-prefix=CHECK-RECUR
# RUN: %{run-test}
#
# RUN: %{record-test}
# REDEFINE: %{lit-args} =
# REDEFINE: %{fc-args} =

# Check that per-test changes to substitutions don't affect other tests in the
# same LIT invocation.
#
# RUN: %{lit} -va %{my-inputs}/shared-substs-*.txt 2>&1 |                      \
# RUN:   FileCheck -check-prefix=SHARED-SUBSTS -match-full-lines %s
#
# SHARED-SUBSTS: shared-substs-0.txt
# SHARED-SUBSTS: GLOBAL: World
# SHARED-SUBSTS: LOCAL0: LOCAL0:Hello LOCAL0:World
# SHARED-SUBSTS: LOCAL0: subst
#
# SHARED-SUBSTS: shared-substs-1.txt
# SHARED-SUBSTS: GLOBAL: World
# SHARED-SUBSTS: LOCAL1: LOCAL1:Hello LOCAL1:World
# SHARED-SUBSTS: LOCAL1: subst
#
# REDEFINE: %{test} = shared-substs-0.txt
# RUN: %{record-test}
# REDEFINE: %{test} = shared-substs-1.txt
# RUN: %{record-test}

# REDEFINE: %{test} = value-equals.txt
# RUN: %{run-and-record-test}

# REDEFINE: %{test} = value-escaped.txt
# RUN: %{run-and-record-test}

# REDEFINE: %{fc-args} = -strict-whitespace
# REDEFINE: %{test} = ws-and-continuations.txt
# RUN: %{run-and-record-test}
# REDEFINE: %{fc-args} =

# Make sure we didn't forget to run something.
#
# RUN: %{lit} --show-tests %{my-inputs} > %t.tests.expected.txt
# RUN: diff -u -w %t.tests.expected.txt %t.tests.actual.txt



# Check the output is expected when tests pass but sanitizer fails.
# Note that there is only one shard which has only one sub-test. However, the summary
# has one pass for the sub-test and one fail for the shard failure due to sanitizer
# reported errors.

# RUN: not %{lit} -v --order=random %{inputs}/googletest-sanitizer-error > %t.out
# FIXME: Temporarily dump test output so we can debug failing tests on
# buildbots.
# RUN: cat %t.out
# RUN: FileCheck < %t.out %s
#
# END.

# CHECK: -- Testing:
# CHECK: FAIL: googletest-sanitizer-error :: [[PATH:[Dd]ummy[Ss]ub[Dd]ir/]][[FILE:OneTest\.py]]/0
# CHECK: *** TEST 'googletest-sanitizer-error :: [[PATH]][[FILE]]/0{{.*}} FAILED ***
# CHECK-NEXT: Script(shard):
# CHECK-NEXT: --
# CHECK-NEXT: GTEST_OUTPUT=json:{{[^[:space:]]*}} GTEST_SHUFFLE=1 GTEST_TOTAL_SHARDS={{[1-6]}} GTEST_SHARD_INDEX=0 GTEST_RANDOM_SEED=123 {{.*}}[[FILE]]
# CHECK-NEXT: --
# CHECK-EMPTY:
# CHECK-EMPTY:
# CHECK:      [ RUN      ] FirstTest.subTestA
# CHECK-NEXT: [       OK ] FirstTest.subTestA (8 ms)
# CHECK:      --
# CHECK-NEXT: exit: 1
# CHECK-NEXT: --
# CHECK:      Failed Tests (1):
# CHECK-NEXT:   googletest-sanitizer-error :: [[PATH]][[FILE]]/0/1
# CHECK: Passed{{ *}}: 1
# CHECK: Failed{{ *}}: 1



# RUN: %{lit} -j 1 -v %{inputs}/test-data-micro --resultdb-output %t.results.out
# RUN: FileCheck < %t.results.out %s
# RUN: rm %t.results.out


# CHECK: {
# CHECK: "__version__"
# CHECK: "elapsed"
# CHECK-NEXT: "tests": [
# CHECK-NEXT:    {
# CHECK-NEXT:      "artifacts": {
# CHECK-NEXT:        "artifact-content-in-request": {
# CHECK-NEXT:          "contents": "VGVzdCBwYXNzZWQu"
# CHECK-NEXT:        }
# CHECK-NEXT:      },
# CHECK-NEXT:      "duration"
# CHECK-NEXT:      "expected": true,
# CHECK-NEXT:      "start_time"
# CHECK-NEXT:      "status": "PASS",
# CHECK-NEXT:      "summary_html": "<p><text-artifact artifact-id=\"artifact-content-in-request\"></p>",
# CHECK-NEXT:      "testId": "test-data-micro :: micro-tests.ini"
# CHECK-NEXT:    },
# CHECK-NEXT:    {
# CHECK-NEXT:      "artifacts": {
# CHECK-NEXT:        "artifact-content-in-request": {
# CHECK-NEXT:          "contents": ""
# CHECK-NEXT:        }
# CHECK-NEXT:      },
# CHECK-NEXT:      "duration"
# CHECK-NEXT:      "expected": true,
# CHECK-NEXT:      "start_time"
# CHECK-NEXT:      "status": "PASS",
# CHECK-NEXT:      "summary_html": "<p><text-artifact artifact-id=\"artifact-content-in-request\"></p>",
# CHECK-NEXT:      "testId": "test-data-micro :: micro-tests.ini:test0microres"
# CHECK-NEXT:    },
# CHECK-NEXT:    {
# CHECK-NEXT:      "artifacts": {
# CHECK-NEXT:        "artifact-content-in-request": {
# CHECK-NEXT:          "contents": ""
# CHECK-NEXT:        }
# CHECK-NEXT:      },
# CHECK-NEXT:      "duration"
# CHECK-NEXT:      "expected": true,
# CHECK-NEXT:      "start_time"
# CHECK-NEXT:      "status": "PASS",
# CHECK-NEXT:      "summary_html": "<p><text-artifact artifact-id=\"artifact-content-in-request\"></p>",
# CHECK-NEXT:      "testId": "test-data-micro :: micro-tests.ini:test1microres"
# CHECK-NEXT:    },
# CHECK-NEXT:    {
# CHECK-NEXT:      "artifacts": {
# CHECK-NEXT:        "artifact-content-in-request": {
# CHECK-NEXT:          "contents": ""
# CHECK-NEXT:        }
# CHECK-NEXT:      },
# CHECK-NEXT:      "duration"
# CHECK-NEXT:      "expected": true,
# CHECK-NEXT:      "start_time"
# CHECK-NEXT:      "status": "PASS",
# CHECK-NEXT:      "summary_html": "<p><text-artifact artifact-id=\"artifact-content-in-request\"></p>",
# CHECK-NEXT:      "testId": "test-data-micro :: micro-tests.ini:test2microres"
# CHECK-NEXT:    }
# CHECK-NEXT: ]
# CHECK-NEXT: }



## Check that we can reorder test runs.

# RUN: cp %{inputs}/reorder/lit_test_times %{inputs}/reorder/.lit_test_times.txt
# RUN: not %{lit-no-order-opt} %{inputs}/reorder > %t.out
# RUN: FileCheck --check-prefix=TIMES < %{inputs}/reorder/.lit_test_times.txt %s
# RUN: FileCheck < %t.out %s
# END.

# TIMES: not-executed.txt
# TIMES-NEXT: subdir/ccc.txt
# TIMES-NEXT: bbb.txt
# TIMES-NEXT: -{{.*}} fff.txt
# TIMES-NEXT: aaa.txt
# TIMES-NEXT: new-test.txt

# CHECK:     -- Testing: 5 tests, 1 workers --
# CHECK-NEXT: FAIL: reorder :: fff.txt
# CHECK-NEXT: PASS: reorder :: subdir/ccc.txt
# CHECK-NEXT: PASS: reorder :: bbb.txt
# CHECK-NEXT: PASS: reorder :: aaa.txt
# CHECK-NEXT: PASS: reorder :: new-test.txt
# CHECK:     Passed: 4



## Show that lit reports the path of tools found via use_llvm_tool.
## Additionally show that use_llvm_tool uses in order of preference:
## 1) The path specified in an environment variable,
## 2) The LLVM tools build directory,
## 3) The PATH, if requested.

# RUN: %{lit} %{inputs}/use-llvm-tool 2>&1 | \
# RUN:   FileCheck %s -DDIR=%p

## The exact breakdown of cases is:
## Case | Env | Build Dir | PATH |
##   1  |  /  |     X     |  N/S | <- Can be found via env
##   2  |  X  |     /     |  N/S | <- Can be found via build dir if env specified
##   3  | N/S |     /     |  N/S | <- Can be found via build dir
##   4  | N/S |     X     |   /  | <- Can be found via PATH, if requested
##   5  | N/S |     X     |  N/S | <- Cannot be found via PATH, if not requested
##   6  |  /  |     /     |   /  | <- Env is preferred over build, PATH
##   7  | N/S |     /     |   /  | <- Build dir is preferred over PATH
##   8  |  X  |     X     |   X  | <- Say nothing if cannot be found if not required
##   9  | N/S |  override |  N/S | <- Use specified search directory, instead of default directory
##  10  | N/S |  override |   /  | <- Use PATH if not in search directory

## Check the exact path reported for the first case, but don't bother for the
## others.
# CHECK:      note: using case1: [[DIR]]{{[\\/]}}Inputs{{[\\/]}}use-llvm-tool{{[\\/]}}env-case1
# CHECK-NEXT: note: using case2: {{.*}}build{{[\\/]}}case2
# CHECK-NEXT: note: using case3: {{.*}}build{{[\\/]}}case3
# CHECK-NEXT: note: using case4: {{.*}}path{{[\\/]}}case4
# CHECK-NOT:  case5
# CHECK-NEXT: note: using case6: {{.*}}env-case6
# CHECK-NEXT: note: using case7: {{.*}}build{{[\\/]}}case7
# CHECK-NOT:  case8
# CHECK-NEXT: note: using case9: {{.*}}search2{{[\\/]}}case9
# CHECK-NEXT: note: using case10: {{.*}}path{{[\\/]}}case10

## Test that if required is True, lit errors if the tool is not found.
# RUN: not %{lit} %{inputs}/use-llvm-tool-required 2>&1 | \
# RUN:   FileCheck %s --check-prefix=ERROR
# ERROR:      note: using found: {{.*}}found
# ERROR-NEXT: fatal: couldn't find 'not-found' program
